// Gavriel State 27jun97
// gav@magmacom.com

// This is JNIT #29.  It performs various patches to improve Jasik's appearance.

#include <Types.h>
#include <Resources.h>
#include <QuickDraw.h>
#include <Folders.h>
#include <Events.h>

#include <SetupA4.h>
#include <A4Stuff.h>
#include <LowMem.h>
#include <Traps.h>


typedef pascal void (*DrawMenuBarProcPtr)(void);

DrawMenuBarProcPtr	gOldDrawMenuBarProc;

pascal void PatchDrawMenuBar(void);
pascal void PatchDrawString(ConstStr255Param s);
pascal short PatchStringWidth(ConstStr255Param s);


// DrawString ProcPtr type definition
typedef pascal void (*DrawStringProcPtr)(ConstStr255Param s);
// StringWidth ProcPtr type definition
typedef pascal short (*StringWidthProcPtr)(ConstStr255Param s);

// Declare a global pointer to save the original traps address into
DrawStringProcPtr		gOldDrawString;
StringWidthProcPtr		gOldStringWidth;

const short	kJasikApplePicture = 834;

//---------------------------------------------------------------------------
// The patch function
//---------------------------------------------------------------------------
pascal void PatchDrawString(ConstStr255Param s)
{
	// Set up A4 globals
	EnterCallback();
	
	// Check for the Apple character
	if (s[1] == '')
	{
		PicHandle	jasikApplePict = ::GetPicture (kJasikApplePicture);
		
		if (jasikApplePict)
		{
			GrafPtr	curPort;
			::GetPort (&curPort);
			
			Rect	jasikPictRect;
			
			jasikPictRect.bottom = curPort->pnLoc.v + 2;
			jasikPictRect.left = curPort->pnLoc.h+16;
			jasikPictRect.top = curPort->pnLoc.v - 10;
			jasikPictRect.right = curPort->pnLoc.h + 16 + 12;
			
			// Draw the icon
			::DrawPicture(jasikApplePict, &jasikPictRect);			
		}
		else
		{
			// Something went wrong.  Call the original
			(*gOldDrawString)("\p");
		}
	}
	else
	{
		// Call the original routine
		(*gOldDrawString)(s);
	}

	// Tear down A4 globals
	ExitCallback();
}

//---------------------------------------------------------------------------
// The patch function
//---------------------------------------------------------------------------
pascal short PatchStringWidth(ConstStr255Param s)
{
	// Set up A4 globals
	EnterCallback();
	
	short	theLength = 0;
	
	// Check for the Apple character
	if (s[1] == '')
	{
		// The icon width is 16 pixels
		theLength = 16;
	}
	else
	{
		// Call the original routine
		theLength = (*gOldStringWidth)(s);
	}
	
	// Tear down A4 globals
	ExitCallback();
	
	return theLength;
}


//---------------------------------------------------------------------------
// Stolen from JasikAppearance, slightly modified
//---------------------------------------------------------------------------
// RetitleAppleMenu to the memory size (using unofficially documented menu list structs)
//---------------------------------------------------------------------------
static void RetitleAppleMenu();
static void SetMenuAppleTitle(MenuHandle hMenu);
static Boolean PStrEqu(const StringPtr a, const StringPtr b);

//---------------------------------------------------------------------------
// Compare two pascal strings
//---------------------------------------------------------------------------


struct MenuRec
{
	MenuHandle menuOH;
	short menuLeft;
};

struct DynamicMenuList
{
	short lastMenu;
	short lastRight;
	short mbResID;
	MenuRec menu[1];
};

static void SetMenuAppleTitle(MenuHandle hMenu)
{	
	short menuNameLen = hMenu[0]->menuData[0];
	
	hMenu[0]->menuData[1] = ''; // 0x14 - the apple char		
	
	for (short count = 2; count <= menuNameLen; count++)
		hMenu[0]->menuData[count] = ' ';
}

static void RetitleAppleMenu()
{
	// Get the apple (first) menu
	DynamicMenuList** hMbar = (DynamicMenuList**)LMGetMenuList();

	if ((hMbar) && (*hMbar))
	{
		
		// Only fiddle if there's a menu in the list
		if(hMbar[0]->lastMenu)
		{
			MenuHandle hAppleMenu = hMbar[0]->menu[0].menuOH;
			if ((hAppleMenu) && (*hAppleMenu))
			{
				// Init the new title		
				SetMenuAppleTitle(hAppleMenu);
			}
		}
	}
}


//---------------------------------------------------------------------------
// The patch function
//---------------------------------------------------------------------------
pascal void PatchDrawMenuBar(void)
{	
	// Set up A4 globals
	EnterCallback();
	
	// Set the Apple menu back to what it's supposed to be.
	RetitleAppleMenu();
	
	// Call the original routine
	(*gOldDrawMenuBarProc)();

	// Aaron's code for drawing the menu bar adornments
	short	theHeight;
	RGBColor	saveColor, theColor = { 0x8888, 0x8888, 0x8888 },
			whiteColor = { 0xFFFF, 0xFFFF, 0xFFFF };
	GrafPtr	savePort, myPort;
	GDHandle	theDevice = GetMainDevice();
	
	theHeight = LMGetMBarHeight();
	if ( theHeight > 0 ) 
	{
		GetPort ( &savePort );
		GetCWMgrPort ( (CGrafPtr *)&myPort );
		SetPort ( myPort );	
		GetForeColor ( &saveColor );
		RGBForeColor ( &theColor );
		MoveTo ( (*theDevice)->gdRect.left, (*theDevice)->gdRect.top + theHeight - 2 );
		LineTo ( (*theDevice)->gdRect.right, (*theDevice)->gdRect.top + theHeight - 2 );
		RGBForeColor ( &whiteColor );
		MoveTo ( (*theDevice)->gdRect.left + 5, (*theDevice)->gdRect.top );
		LineTo ( (*theDevice)->gdRect.right - 6, (*theDevice)->gdRect.top );
		RGBForeColor ( &saveColor );
		SetPort ( savePort );
	}
	
	// Tear down A4 globals
	ExitCallback();

}



//---------------------------------------------------------------------------
// The INIT main and patch installer.
//---------------------------------------------------------------------------
int main(void)
{
	Handle H;
	
	EnterCodeResource();	// Set up for globals	
	PrepareCallback();
	
	// Set up the patch - the INIT *must* be in the Sysheap
	H = GetResource('JNIT',29);
	if (H) 
	{
		// Detach and lock the patch
		DetachResource(H);
		HLock(H);
		HNoPurge(H);

		// Get the old address
		gOldDrawMenuBarProc = (DrawMenuBarProcPtr)NGetTrapAddress(_DrawMenuBar, ToolTrap);
		
		// Install the patch
		NSetTrapAddress((UniversalProcPtr)&PatchDrawMenuBar, _DrawMenuBar, ToolTrap);


		// Get the old address
		gOldDrawString = (DrawStringProcPtr)NGetTrapAddress(_DrawString, ToolTrap);
		
		// Install the patch
		NSetTrapAddress((UniversalProcPtr)&PatchDrawString, _DrawString, ToolTrap);

		// Get the old address
		gOldStringWidth = (StringWidthProcPtr)NGetTrapAddress(_StringWidth, ToolTrap);
		
		// Install the patch
		NSetTrapAddress((UniversalProcPtr)&PatchStringWidth, _StringWidth, ToolTrap);

	}
	
	ExitCodeResource();		// Shutdown use of globals (A4).
}