// GrowArrangement.cp

#ifndef GrowArrangement_h
#include "GrowArrangement.h"
#endif
#ifndef MinMax_h
#include "MinMax.h"
#endif

GrowArrangement::GrowArrangement( WindowFocus& focus )
  : grow( focus )
  {
	RightLine().SetView( rightLine );
	BottomLine().SetView( bottomLine );
	Grow().SetView( grow );
  }

void GrowArrangement::Arrange( Rectangle bounds )
  {
	Rectangle grow( bounds.right - barSize,
						 bounds.bottom - barSize,
						 bounds.right,
						 bounds.bottom );

	Rectangle main( bounds.left,
						 bounds.top,
						 grow.left,
						 grow.top );
	
	Rectangle rightLine( grow.left,
								bounds.top,
								grow.left + 1,
								grow.top );
	
	Rectangle rightBar( rightLine.right,
							  bounds.top,
							  bounds.right,
							  grow.top );
	
	Rectangle bottomLine( bounds.left,
								 grow.top,
								 grow.left,
								 grow.top + 1 );
	
	Rectangle bottomBar( bounds.left,
								bottomLine.bottom,
								grow.left,
								bounds.bottom );
	
	Main().SetBounds( main );
	Right().SetBounds( rightBar );
	Bottom().SetBounds( bottomBar );
	RightLine().SetBounds( rightLine );
	BottomLine().SetBounds( bottomLine );
	Grow().SetBounds( grow );
  }

uint16 GrowArrangement::MinimumHeight() const
  {
	uint16 size = Max( Main().Size().MinimumHeight(),
							 Right().Size().MinimumHeight() );
	
	Assert( size <= maxint16 - barSize );
	
	return size + barSize;
  }

uint16 GrowArrangement::MinimumWidth() const
  {
	uint16 size = Max( Main().Size().MinimumWidth(),
							 Bottom().Size().MinimumWidth() );
	
	Assert( size <= maxint16 - barSize );
	
	return size + barSize;
  }

uint16 GrowArrangement::MaximumHeight() const
  {
	uint16 size = Min( Main().Size().MaximumHeight(),
							 Right().Size().MaximumHeight() );
	
	if ( size >= maxint16 - barSize )
		return maxint16;
	
	return size + barSize;
  }

uint16 GrowArrangement::MaximumWidth() const
  {
	uint16 size = Min( Main().Size().MaximumWidth(),
							 Bottom().Size().MaximumWidth() );
	
	if ( size >= maxint16 - barSize )
		return maxint16;
	
	return size + barSize;
  }

uint16 GrowArrangement::ReasonableHeight() const
  {
	uint16 size = Max( Main().Size().ReasonableHeight(),
							 Right().Size().ReasonableHeight() );
	
	Assert( size <= maxint16 - barSize );
	
	return size + barSize;
  }

uint16 GrowArrangement::ReasonableWidth() const
  {
	uint16 size = Max( Main().Size().ReasonableWidth(),
							 Bottom().Size().ReasonableWidth() );
	
	Assert( size <= maxint16 - barSize );
	
	return size + barSize;
  }

uint16 GrowArrangement::BestHeight() const
  {
	uint16 size = Main().Size().BestHeight();
	
	size = Min( size, Right().Size().MaximumHeight() );
	size = Max( size, Right().Size().MinimumHeight() );
	
	Assert( size <= maxint16 - barSize );
	
	return size + barSize;
  }

uint16 GrowArrangement::BestHeight( uint16 bound ) const
  {
	if ( bound <= barSize )
		return bound;
	
	bound -= barSize;

	uint16 size = Main().Size().BestHeight( bound );
	
	size = Min( size, Right().Size().MaximumHeight() );
	size = Max( size, Right().Size().MinimumHeight() );
	size = Min( size, bound );
	
	Assert( size <= maxint16 - barSize );
	
	return size + barSize;
  }

uint16 GrowArrangement::BestWidth() const
  {
	uint16 size = Main().Size().BestWidth();
	
	size = Min( size, Bottom().Size().MaximumWidth() );
	size = Max( size, Bottom().Size().MinimumWidth() );
	
	Assert( size <= maxint16 - barSize );
	
	return size + barSize;
  }

uint16 GrowArrangement::BestWidth( uint16 bound ) const
  {
	if ( bound <= barSize )
		return bound;
	
	bound -= barSize;
	
	uint16 size = Main().Size().BestWidth( bound );
	
	size = Min( size, Bottom().Size().MaximumWidth() );
	size = Max( size, Bottom().Size().MinimumWidth() );
	size = Min( size, bound );
	
	Assert( size <= maxint16 - barSize );
	
	return size + barSize;
  }
